#
# Movie selection interface rewritten as an additional plugin by moveq
# Copyright (C) 2007-2008 moveq / Suomipoeka team (suomipoeka@gmail.com)
# In case of reuse of this source code please do not remove this copyright.
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	For more information on the GNU General Public License see:
#	<http://www.gnu.org/licenses/>.
#
from Components.config import *
from Components.GUIComponent import GUIComponent
from Components.MultiContent import MultiContentEntryText, MultiContentEntryPixmapAlphaTest
#from Tools.Directories import SCOPE_SKIN_IMAGE
from time import localtime
from enigma import eTimer, eListboxPythonMultiContent, eListbox, gFont, iServiceInformation, iRecordableService, \
	RT_HALIGN_LEFT, RT_HALIGN_RIGHT, RT_HALIGN_CENTER, eServiceReference, getBestPlayableServiceReference, eServiceCenter, loadPNG

from RecordingsControl import RecordingsControl
from DelayedFunction import DelayedFunction
from SuomipoekaTasker import spDebugOut
from Suomipoeka import _ 

import NavigationInstance
import os


###############
#from Components.Converter.EventTime import EventTime
#from Components.Converter.ServiceEvent import ServiceEvent
#class RecordProgress(EventTime):
#	def __init__(self, service):
#		EventTime.__init__(self, EventTime.PROGRESS)
#		self.source = ServiceEvent()
#		self.source.newService(service)
###############


currentSelectionCount = 0
class MovieList(GUIComponent):
	def __init__(self):
		self.list = [] # to avoid blue screen when movie list opened with long VIDEO-press
		GUIComponent.__init__(self)
		self.l = eListboxPythonMultiContent()
		self.l.setFont(0, gFont("Regular", 22))
		self.l.setFont(1, gFont("Regular", 20))
		self.l.setFont(2, gFont("Regular", 16))
		self.l.setBuildFunc(self.buildMovieListEntry)
		self.l.setItemHeight(25)
		self.alphaSort = False
		self.newRecordings = False
		self.selectionList = None
		self.recControl = RecordingsControl(self.recStateChange)
		self.highlightsMov = []
		self.highlightsDel = []
		self.backPic = loadPNG('/usr/lib/enigma2/python/Plugins/Extensions/Suomipoeka/img/back.png')
		self.dirPic = loadPNG('/usr/lib/enigma2/python/Plugins/Extensions/Suomipoeka/img/dir.png')
		self.mov1Pic = loadPNG('/usr/lib/enigma2/python/Plugins/Extensions/Suomipoeka/img/movie_yes.png')
		self.mov2Pic = loadPNG('/usr/lib/enigma2/python/Plugins/Extensions/Suomipoeka/img/movie.png')
		self.mov3Pic = loadPNG('/usr/lib/enigma2/python/Plugins/Extensions/Suomipoeka/img/movie_red.png')
		self.mp3Pic = loadPNG('/usr/lib/enigma2/python/Plugins/Extensions/Suomipoeka/img/music.png')
		self.vlcPic = loadPNG('/usr/lib/enigma2/python/Plugins/Extensions/Suomipoeka/img/vlc.png')
		self.vlcdPic = loadPNG('/usr/lib/enigma2/python/Plugins/Extensions/Suomipoeka/img/vlcdir.png')
		self.loadPath = config.suomipoeka.movie_homepath.value

	def recStateChange(self, recList):
		try:
			DelayedFunction(100, self.reload, self.loadPath)
			#spDebugOut("[spML] recStateChange...")
		except Exception, e:
			spDebugOut("[spML] recStateChange exception:\n" + str(e))

	def setAlphaSort(self, trueOrFalse):
		self.alphaSort = trueOrFalse

	def buildMovieListEntry(self, serviceref, sortkey, datesort, moviestring, filename, selnum):
		if datesort is None:
			res = [ None ]
			pmap = [self.dirPic, self.vlcdPic][sortkey=="VLCd" or filename=="VLC servers"]
			if filename=="..": pmap = self.backPic
			if config.suomipoeka.movie_dateleft.value:
				noicon = (not config.suomipoeka.movie_icons.value) * 20
				res.append(MultiContentEntryPixmapAlphaTest(pos=(8,2), size=(22,22), png=pmap, **{}))
				res.append(MultiContentEntryText(pos=(92-noicon, 0), size=(310, 22), font=1, flags=RT_HALIGN_LEFT, text=filename))
#				res.append(MultiContentEntryText(pos=(485, 0), size=(24, 22), font=1, flags=RT_HALIGN_LEFT, text=selnumtxt))
			else:
#				res.append(MultiContentEntryText(pos=(0, 0), size=(24, 22), font=1, flags=RT_HALIGN_LEFT, text=selnumtxt))
				res.append(MultiContentEntryPixmapAlphaTest(pos=(26,2), size=(22,22), png=pmap, **{}))
				res.append(MultiContentEntryText(pos=(50, 0), size=(310, 22), font=1, flags=RT_HALIGN_LEFT, text=filename))
				res.append(MultiContentEntryText(pos=(366, 0), size=(200, 22), font=1, flags=RT_HALIGN_LEFT, text=_("Directory")))
			return res
		if serviceref.flags & eServiceReference.mustDescent:
			return None

		date, time, pixmap = "---.---  ", "---:---", self.mov1Pic
		nameWithPath = serviceref.getPath()
		if config.suomipoeka.movie_mark.value and not os.path.exists(nameWithPath + ".cuts"):
			pixmap = self.mov2Pic

		if datesort == "VLCs":
			date, time, pixmap = "VLC", " stream", self.vlcPic
		elif not datesort.startswith("0000"):
			# datesort = YYMMDDTTTT
			time = datesort[6:8] + ":" + datesort[8:10]
			date = datesort[4:6] + "." + datesort[2:4] + "." + "  " #+ datesort[0:2]
		elif filename.endswith(".mp3"):
			date, time, pixmap = "MP3", " audio", self.mp3Pic
			moviestring = moviestring[:-4]
		elif filename.endswith(".mpg"):
			date, time = "MPEG", " video"
			moviestring = moviestring[:-4]
		elif filename.endswith(".mpeg"):
			date, time = "MPEG", " video"
			moviestring = moviestring[:-5]

		selnumtxt = ""
		if selnum == 9999: selnumtxt = "[M]"
		elif selnum == 9998: selnumtxt = "[D]"
		elif selnum > 0: selnumtxt = "%02d" % selnum
		if serviceref in self.highlightsMov: selnumtxt = "[M]"
		elif serviceref in self.highlightsDel: selnumtxt = "[D]"

		if self.recControl.isRecording(nameWithPath):
			date, pixmap = "Rec...  ", self.mov3Pic

		res = [ None ]
		if config.suomipoeka.movie_dateleft.value:
			posx = 2
			if config.suomipoeka.movie_icons.value:
				posx = 22
				res.append(MultiContentEntryPixmapAlphaTest(pos=(0,2), size=(20,20), png=pixmap, **{}))
			res.append(MultiContentEntryText(pos=(posx, 0), size=(60, 22), font=1, flags=RT_HALIGN_LEFT, text=date))
			res.append(MultiContentEntryText(pos=(70+posx, 0), size=(420, 22), font=1, flags=RT_HALIGN_LEFT, text=moviestring))
			res.append(MultiContentEntryText(pos=(495, 0), size=(24, 22), font=1, flags=RT_HALIGN_CENTER, text=selnumtxt))
		else:
			posx = 26
			res.append(MultiContentEntryText(pos=(0, 0), size=(26, 22), font=1, flags=RT_HALIGN_LEFT, text=selnumtxt))
			if config.suomipoeka.movie_icons.value:
				posx = 48
				res.append(MultiContentEntryPixmapAlphaTest(pos=(26,2), size=(20,20), png=pixmap, **{}))
			res.append(MultiContentEntryText(pos=(380, 0), size=(90, 22), font=1, flags=RT_HALIGN_RIGHT, text=date))
			res.append(MultiContentEntryText(pos=(posx, 0), size=(400-posx, 22), font=1, flags=RT_HALIGN_LEFT, text=moviestring))
			res.append(MultiContentEntryText(pos=(470, 0), size=(85, 22), font=1, flags=RT_HALIGN_LEFT, text=time))
		return res

	def moveToIndex(self, index):
		self.instance.moveSelectionTo(index)

	def getCurrentIndex(self):
		return self.instance.getCurrentIndex()

	def getCurrentEvent(self):
		l = self.l.getCurrentSelection()
		return l and l[0] and self.serviceHandler.info(l[0]).getEvent(l[0])

	def getCurrent(self):
		l = self.l.getCurrentSelection()
		return l and l[0]

	GUI_WIDGET = eListbox
	
	def postWidgetCreate(self, instance):
		instance.setWrapAround(True)
		instance.setContent(self.l)

	def removeService(self, service):
		for l in self.list[:]:
			if l[0] == service:
				self.list.remove(l)
		self.l.setList(self.list)

	def serviceBusy(self, service):
		return service in self.highlightsMov or service in self.highlightsDel

	def serviceMoving(self, service):
		return service in self.highlightsMov

	def serviceDeleting(self, service):
		return service in self.highlightsDel

	def highlightService(self, enable, mode, service):
		if enable:
			if mode == "move":
				self.highlightsMov.append(service)
				self.toggleSelection(service, 9999)
			elif mode == "del":
				self.highlightsDel.append(service)
				self.toggleSelection(service, 9998)
		else:
			if mode == "move":
				self.highlightsMov.remove(service)
			elif mode == "del":
				self.highlightsDel.remove(service)

	def __len__(self):
		return len(self.list)

	def makeSelectionList(self):
		selList = [ ]
		global currentSelectionCount
		if currentSelectionCount == 0:
			# if no selections made, select the current cursor position
			single = self.l.getCurrentSelection()
			if single:
				selList.append(single[0])
		else:
			selList = self.selectionList
		return selList

	def resetSelectionList(self):
		self.selectionList = None
		global currentSelectionCount
		currentSelectionCount = 0

	def multiSelectIdx(self, index):
		self.toggleSelection(self.list[index][0])

	def toggleSelection(self, service=None, overrideNum=None):
		global currentSelectionCount
		x, index = None, 0
		if service is None:
			if self.l.getCurrentSelection() is None: return
			index = self.getCurrentIndex()
			x = self.list[index]
		else:
			for e in self.list:
				if e[0] == service:
					x = e
					break
				index += 1

		if x is None: return
		if self.selectionList == None:
			self.selectionList = [ ]

		newselnum = x[5]	# init with old selection number
		if overrideNum == None:
			if self.serviceBusy(x[0]): return	# no toggle if file being operated on
			# basic selection toggle
			if newselnum == 0:
				# was not selected
				currentSelectionCount += 1
				newselnum = currentSelectionCount
				self.selectionList.append(x[0]) # append service
			else:
				# was selected, reset selection number and decrease all that had been selected after this
				newselnum = 0
				currentSelectionCount -= 1
				count = 0
				if x is not None:
					self.selectionList.remove(x[0]) # remove service
				for i in self.list:
					if i[5] > x[5]:
						self.list.remove(self.list[count])
						self.list.insert(count, (i[0],i[1],i[2],i[3],i[4],i[5]-1))
						self.l.invalidateEntry(count) # force redraw
					count += 1
		else:
			newselnum = overrideNum * (newselnum == 0)

		self.list.remove(self.list[index])
		self.list.insert(index, (x[0],x[1],x[2],x[3],x[4],newselnum))
		self.l.invalidateEntry(index) # force redraw of the modified item

	def invalidateService(self, service):
		i = 0
		for entry in self.list:
			if entry[0] == service:
				self.l.invalidateEntry(i) # force redraw of the item
				break
			i += 1

	def invalidateCurrent(self):
		self.l.invalidateEntry(self.getCurrentIndex())

	def getLenghtOfCurrent(self):
		ref = self.getCurrent()
		return (self.serviceHandler.info(ref).getLength(ref)+30)/60

	def getFileNameOfService(self, service):
		for entry in self.list:
			if entry[0] == service:
				return entry[4]
		return None

	def createDirlist(self, loadPath):
		subdirlist = []
		dirlist = os.listdir(loadPath)	# only need to deal with spaces when executing in shell
		# add sub directories to the list
		for p in dirlist:
			if os.path.isdir(loadPath + p):
				if config.suomipoeka.movie_trashcan_hide.value:
					# TODO: dereference links for comparison?
					if (loadPath + p) == config.suomipoeka.movie_trashpath.value: continue
				subdirlist.append(p)
		subdirlist.sort(key=str.lower)
		if loadPath != "/" and loadPath[:-1] != config.suomipoeka.movie_pathlimit.value:
			subdirlist.insert(0, "..")
		if loadPath[:-1] == config.suomipoeka.movie_homepath.value:
			if os.path.exists("/usr/lib/enigma2/python/Plugins/Extensions/VlcPlayer"):
				subdirlist.insert(0, "VLC servers")
		return subdirlist

	def reloadVlcServers(self):
		try:
			from Plugins.Extensions.VlcPlayer.VlcServerConfig import vlcServerConfig
			self.vlcServers = vlcServerConfig.getServerlist()	# settings change requires running this
			self.list = []
			sref = eServiceReference("2:0:1:0:0:0:0:0:0:0:" + "..")	# dummy ref
			self.list.append((sref, None, None, None, "..", None))
			for srv in self.vlcServers:
				srvName = srv.getName()
				sref = eServiceReference("2:0:1:0:0:0:0:0:0:0:" + srvName)	# dummy ref
				self.list.append((sref, "VLCd", None, None, srvName, None))
			self.list.sort(key=lambda x: x[1],reverse=False)
			self.l.setList(self.list)
		except:	pass

	def reloadVlcFilelist(self):
		self.list = []
		sref = eServiceReference("2:0:1:0:0:0:0:0:0:0:" + "..")	# dummy ref
		self.list.append((sref, None, None, None, "..", None))

		vlcPath = self.loadPath[self.loadPath.find("VLC servers/")+12:]	# server/dir/name/
		serverName = vlcPath.split("/")[0]
		spDebugOut("[spML] path on %s = %s" %(serverName, vlcPath))
		server = None
		self.vlcServer = None
		for srv in self.vlcServers:
			if srv.getName() == serverName: server = srv	# find the server
		if server is not None:
			try:
				self.vlcServer = server
				spDebugOut("[spML] baseDir = " + server.getBasedir())
				vlcPath = vlcPath[len(serverName):]
				spDebugOut("[spML] vlcPath = " + vlcPath)
				(vlcFiles, vlcDirs) = server.getFilesAndDirs(server.getBasedir() + vlcPath, None)
				spDebugOut("[spML] got files and dirs...")
				for d in vlcDirs:
					if d[0] == "..": continue
					sref = eServiceReference("2:0:1:0:0:0:0:0:0:0:" + d[0])	# dummy ref
					self.list.append((sref, "VLCd", None, None, d[0], None))
				for f in vlcFiles:
					from Plugins.Extensions.VlcPlayer.VlcFileList import MEDIA_EXTENSIONS	# , VlcFileListEntry
#					others = ["mp4", "mpeg4", "mkv"]
#					extension = f[0].split('.')[-1].lower()
#					if MEDIA_EXTENSIONS.has_key(extension) or extension in others:
					sref = eServiceReference("2:0:1:0:0:0:0:0:0:0:" + f[0])	# dummy ref
					self.list.append((sref, None, "VLCs", f[0], f[1], None))
			except Exception, e:
				spDebugOut("[spML] reloadVlcFilelist exception:\n" + str(e))
		#self.list.sort(key=lambda x: x[1],reverse=False)
		self.l.setList(self.list)

	def reload(self, loadPath):
		global currentSelectionCount
		currentSelectionCount = 0
		self.selectionList = None
		self.list = []
		self.newRecordings = False
		self.loadPath = loadPath

		spDebugOut("[spML] LOAD PATH:\n" + str(loadPath))
		if loadPath.endswith("VLC servers/"):
			self.reloadVlcServers()
			return
		elif loadPath.find("VLC servers/")>0:
			self.reloadVlcFilelist()
			return

		subdirlist = self.createDirlist(loadPath)
		root = eServiceReference("2:0:1:0:0:0:0:0:0:0:" + loadPath)
		self.serviceHandler = eServiceCenter.getInstance()
		list = self.serviceHandler.list(root)

		while 1:
			serviceref = list.getNext()
			if not serviceref.valid(): break
			if serviceref.flags & eServiceReference.mustDescent: continue
			info = self.serviceHandler.info(serviceref)
			if info is None: continue

			moviestring = ""
			mpath = serviceref.getPath()
			filename = mpath.split("/")[-1]
			date = "0000"
			if filename[0:8].isdigit() and filename[9:13].isdigit() and not filename[8:1].isdigit():
				date = filename[2:8] + filename[9:13]
			else:
				# files that do not have a date, sort them alphabetically
				if config.suomipoeka.movielist_reversed.value:
					date += filename.lower()
				else:
					for x in filename.lower(): date += chr(255-ord(x))

# > meta load
			if os.path.exists(mpath + ".meta") and config.suomipoeka.movie_metaload.value:
				file = open(mpath + ".meta", "r")
				file.readline()
				line = file.readline()
				file.close()
				moviestring = line.rstrip("\r\n")
			else:
# < meta load
				moviestring = filename.replace(".ts","").replace("_ "," ").replace("_"," ")
				if moviestring[0:8].isdigit() and moviestring[9:13].isdigit() and not moviestring[8:1].isdigit():
					moviestring = moviestring[16:]	# skips "YYYYMMDD TIME - "
					chlMarker = moviestring.find(" - ")
					if chlMarker > 0:
						moviestring = moviestring[3+chlMarker:]

			sortkey = None
			if self.alphaSort: sortkey = moviestring[:16] + date

			if not (self.serviceMoving(serviceref) and config.suomipoeka.movie_hide_mov.value):
				if not (self.serviceDeleting(serviceref) and config.suomipoeka.movie_hide_del.value):
					# TODO: cleanup table structure
					# (service, sortkey, sortdate, sortstring, filename, selnum)
					self.list.append((serviceref, sortkey, date, moviestring, filename, 0))

		if self.alphaSort:
			self.list.sort(key=lambda x: x[1],reverse=config.suomipoeka.movielist_reversed.value)
		else:
			self.list.sort(key=lambda x: x[2],reverse=not config.suomipoeka.movielist_reversed.value)

		# add sub directories to the list
		if subdirlist is not None:
			dcount = 0
			for dirp in subdirlist:
				sref = eServiceReference("2:0:1:0:0:0:0:0:0:0:" + dirp)	# dummy ref
				self.list.insert(dcount, (sref, None, None, None, dirp, 0))
				dcount += 1
		self.l.setList(self.list)
		self.resetSelectionList()

	def currentSelIsVlc(self):
		try:	return self.list[self.getCurrentIndex()][2] == "VLCs"
		except:	return False

	def currentSelIsDirectory(self):
		try:	return self.list[self.getCurrentIndex()][2] is None or self.list[self.getCurrentIndex()][2] == "VLCd"
		except:	return False

	def getCurrentSelDir(self):
		return self.list[self.getCurrentIndex()][4]

	def moveTo(self, serviceref):
		found = 0
		count = 0
		for x in self.list:
			if x[0] == serviceref:
				found = count
			count += 1
		self.instance.moveSelectionTo(found)

	def moveDown(self):
		self.instance.moveSelection(self.instance.moveDown)
