#
# New movie player written by moveq
# Copyright (C) 2007-2008 moveq / Suomipoeka team (suomipoeka@gmail.com)
# In case of reuse of this source code please do not remove this copyright.
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	For more information on the GNU General Public License see:
#	<http://www.gnu.org/licenses/>.
#
from enigma import eTimer, iPlayableService
from Components.config import *
from Components.ActionMap import HelpableActionMap
from Components.ServiceEventTracker import ServiceEventTracker
from Screens.Screen import Screen
from Screens.InfoBarGenerics import InfoBarSeek
from Screens.InfoBar import MoviePlayer, InfoBar
from Screens.MessageBox import MessageBox

from Suomipoeka import _ 
from SuomipoekaTasker import spDebugOut

import os

class MoviePlayerSP(MoviePlayer):
	ENABLE_RESUME_SUPPORT = True
	ALLOW_SUSPEND = True

	def __init__(self, session, playlist, recordings):
		# MoviePlayer.__init__ will start playback for the given service!!
		MoviePlayer.__init__(self, session, session.nav.getCurrentlyPlayingServiceReference())
		self.skinName = "MoviePlayer"

		self["actions"] = HelpableActionMap(self, "PluginPlayerActions",
			{
				"nextBouquet":	(self.nextBouquet,	_("Advanced skip forward")),
				"prevBouquet":	(self.prevBouquet,	_("Advanced skip backward")),
				"leavePlayer":	(self.leavePlayer,	_("Stop playback")),
				"bRADIO":		(self.btnRadio, 	_("Open extensions menu"))
			})

		# set tracker for the End Of File event
		self.__event_tracker = ServiceEventTracker(screen=self, eventmap=
			{
				iPlayableService.evEOF: self.evEOF,
			})

		self.seekTimer = eTimer()
		self.seekTimer.timeout.get().append(self.seekTimeout)
		self.seekTimeout()

		self.playInit(playlist, recordings)
		self.onShown.append(self.onDialogShow)

	def playInit(self, playlist, recordings):
		self.lastservice = self.session.nav.getCurrentlyPlayingServiceReference()
		self.playlist = playlist
		self.playcount = -1
		self.recordings = recordings
		self.firstStart = False
		self.movieSelectionOpen = False
		self.curretlyPlaying = None
		self.recordings.setPlayerInstance(self)
		self.seekTimeout()

	def onDialogShow(self):
		if self.firstStart:
			return
		self.firstStart = True
		self.evEOF()	# begin playback

	def btnRadio(self):
		try:
			InfoBar.instance.showExtensionSelection()
		except Exception, e:
			spDebugOut("[spPlayer] btnRadio exception:\n" + str(e))

	def seekTimeout(self):
		self.seekTimer.stop()
		self.seekCount = config.suomipoeka.player_seek.value * 90000
		self.seekDirection = 0

	def seekTimein(self):
		self.seekTimer.stop()
		self.seekTimer.start(3000, False)

	def advancedSeek(self, dir1, dir2):
		try:
			self.seekTimein()
			if self.seekDirection == dir1:
				pass
			elif self.seekDirection == dir2:
				self.seekCount = self.seekCount * config.suomipoeka.player_seekchg.value / 100
			self.seekDirection = dir1
			InfoBarSeek.doSeekRelative(self, self.seekDirection * self.seekCount)
		except:
			self.session.open(MessageBox, _("Failed to access Enigma2's seek function."), MessageBox.TYPE_ERROR)

	def nextBouquet(self):
		self.advancedSeek(1, -1)

	def prevBouquet(self):
		self.advancedSeek(-1, 1)

	def evEOF(self, needToClose=False):
		# see if there are more to play
		if (self.playcount+1) < len(self.playlist):
			self.is_closing = False
			self.playcount += 1
			service = self.playlist[self.playcount]
			self.curretlyPlaying = service
			if os.path.exists(service.getPath()):
				self.recordings.viewedToggle(service, True)		# rename .cutsr to .cuts if user has toggled it
				self.session.nav.playService(service)
				self.currentlyPlaying = service
				self.setSeekState(InfoBarSeek.SEEK_STATE_PLAY)
				self.doSeek(0)
			else:
				self.session.open(MessageBox, _("Skipping movie, the file does not exist.\n\n") + service.getPath(), MessageBox.TYPE_ERROR, 10)
				self.evEOF()
		else:
			if needToClose or config.usage.on_movie_eof.value != "pause":	# ask, movielist, quit or pause
				self.curretlyPlaying = None
				if not self.movieSelectionOpen: # if True then close() is called in movieSelected()
					self.stopPlayback()
					self.recordings.setPlayerInstance(None, True)	# mayReopen = True
					self.close()
					spDebugOut("[spPlayer] Player closed by EOF")
				else:
					self.is_closing = True
					self.stopPlayback()

	def leavePlayer(self):
		self.stopPlayback()
		self.recordings.setPlayerInstance(None, True)	# mayReopen = True
		self.close()
		spDebugOut("[spPlayer] Player closed by user")

	def stopPlayback(self):
		try:
			self.session.nav.playService(self.lastservice)
		except Exception, e:
			spDebugOut("[spPlayer] TV service fallback exception:\n" + str(e))

	def showMovies(self):
		try:
			self.movieSelectionOpen = True
			self.session.execDialog(self.recordings)
		except Exception, e:
			spDebugOut("[spPlayer] showMovies exception:\n" + str(e))

	def removeFromPlaylist(self, deletedlist):
		callEOF = False
		for x in deletedlist:
			if x == self.curretlyPlaying:
				callEOF = True
			if x in self.playlist:
				self.playlist.remove(x)
		if callEOF:
			self.evEOF(True)	# force playback of the next movie or close the player if none left

	def currentlyPlayedMovie(self):
		return self.curretlyPlaying

	def movieSelected(self, playlist):
		self.movieSelectionOpen = False
		if playlist is not None and len(playlist) > 0:
			self.playcount = -1
			self.playlist = playlist
			self.evEOF()	# start playback of the first movie
		else:
			if self.is_closing:
				self.recordings.setPlayerInstance(None, False)	# mayReopen = False
				self.close()
				spDebugOut("[spPlayer] Delayed player close by EOF")

