#
# Movie selection interface rewritten as an additional plugin by moveq
# Copyright (C) 2007-2008 moveq / Suomipoeka team (suomipoeka@gmail.com)
# In case of reuse of this source code please do not remove this copyright.
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	For more information on the GNU General Public License see:
#	<http://www.gnu.org/licenses/>.
#
from Components.ActionMap import ActionMap, HelpableActionMap
from Components.Button import Button
from Components.config import *
from Components.Label import Label
from Components.Pixmap import Pixmap
from Components.Sources.ServiceEvent import ServiceEvent
from Screens.Screen import Screen
from Screens.HelpMenu import HelpableScreen
from Screens.MessageBox import MessageBox
from Tools import Notifications
from time import localtime

from enigma import eServiceReference, eServiceCenter, eTimer, eSize
import os

from MovieList import MovieList
from MovieSelectionMenu import MovieMenu
from MoviePlayer import MoviePlayerSP
from SuomipoekaTasker import spTasker, spDebugOut
from Suomipoeka import _, SuomipoekaAutoremove
from DelayedFunction import DelayedFunction

LastPlayedMovies = None


def mountpoint(path, first=True):
	if first: path = os.path.realpath(path)
	if os.path.ismount(path) or len(path)==0: return path
	return mountpoint(os.path.split(path)[0], False)


class VlcFileListWrapper:
	def __init__(self):
		pass
	def getNextFile(self):
		return None, None
	def getPrevFile(self):
		return None, None


class SelectionEventInfo:
	def __init__(self):
		self["DescriptionBorder"] = Pixmap()
		self["Service"] = ServiceEvent()
		self["FileSize"] = Label("")
		self.prev_descdis = not config.suomipoeka.movie_descdis.value

	def updateEventInfo(self):
		if not config.suomipoeka.movie_descdis.value:
			try:
				if self["list"].currentSelIsDirectory() or self["list"].currentSelIsVlc():
					self["FileSize"].setText("")
					self["Service"].newService(None)
				else:
					serviceref = self.getCurrent()
					self["FileSize"].setText("(%d MB)" %(os.path.getsize(serviceref.getPath())/1048576))
					self["Service"].newService(serviceref)
			except Exception, e:
				spDebugOut("[spML] updateEventInfo exception:\n" + str(e))

	def descFieldInit(self):
		if self.prev_descdis == config.suomipoeka.movie_descdis.value: return
		self.prev_descdis = config.suomipoeka.movie_descdis.value

		if config.suomipoeka.movie_descdis.value:
			width = self["list"].instance.size().width()
			height = self.WINDOW_HEIGHT - self["key_red"].instance.size().height() - 2
			self["list"].instance.resize(eSize(width, height))
			self["DescriptionBorder"].hide()
		else:
			width = self["list"].instance.size().width()
			height = self.WINDOW_HEIGHT - self["key_red"].instance.size().height() - self["DescriptionBorder"].instance.size().height()
			self["list"].instance.resize(eSize(width, height))
			self["DescriptionBorder"].show()
			self.updateEventInfo()


class MovieSelectionSP(Screen, HelpableScreen, SelectionEventInfo):
	WINDOW_HEIGHT = 444

	skin = """
	<screen name="MovieSelectionSP" position="80,80" size="560,448" title=" ">
		<ePixmap name="red" position="0,0" zPosition="4" size="140,40" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/Suomipoeka/img/key-red.png" transparent="1" alphatest="on" />
		<ePixmap name="green" position="140,0" zPosition="4" size="140,40" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/Suomipoeka/img/key-green.png" transparent="1" alphatest="on" />
		<ePixmap name="yellow" position="280,0" zPosition="4" size="140,40" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/Suomipoeka/img/key-yellow.png" transparent="1" alphatest="on" />
		<ePixmap name="blue" position="420,0" zPosition="4" size="140,40" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/Suomipoeka/img/key-blue.png" transparent="1" alphatest="on" />
		<widget name="key_red" position="0,0" zPosition="5" size="140,40" valign="center" halign="center" font="Regular;21" transparent="1" foregroundColor="#ffffff" shadowColor="#000000" shadowOffset="-1,-1" />
		<widget name="key_green" position="140,0" zPosition="5" size="140,40" valign="center" halign="center" font="Regular;21" transparent="1" foregroundColor="#ffffff" shadowColor="#000000" shadowOffset="-1,-1" />
		<widget name="key_yellow" position="280,0" zPosition="5" size="140,40" valign="center" halign="center"  font="Regular;21" transparent="1" foregroundColor="#ffffff" shadowColor="#000000" shadowOffset="-1,-1" />
		<widget name="key_blue" position="420,0" zPosition="5" size="140,40" valign="center" halign="center" font="Regular;21" transparent="1" foregroundColor="#ffffff" shadowColor="#000000" shadowOffset="-1,-1" />
		<widget name="list" position="0,42" size="560,300" zPosition="2" backgroundColor="#270b1b1c" scrollbarMode="showOnDemand" />

		<widget name="DescriptionBorder" position="0,345" size="560,103" pixmap="skin_default/border_eventinfo.png" zPosition="3" transparent="1" alphatest="on" />

		<widget source="Service" render="Label" position="5,348" size="550,22" foregroundColor="#00bab329" backgroundColor="#270b1b1c" font="Regular;18" shadowColor="#000000" shadowOffset="-2,-2" zPosition="1" halign="left" >
			<convert type="MovieInfo">RecordServiceName</convert>
		</widget>

		<widget source="Service" render="Label" position="5,369" size="550,57" font="Regular;16" foregroundColor="#cccccc" backgroundColor="#270b1b1c" zPosition="1" >
			<convert type="MovieInfo">ShortDescription</convert>
		</widget>

		<widget source="Service" render="Label" position="5,424" size="275,19" foregroundColor="#00bab329" backgroundColor="#270b1b1c" font="Regular;17" shadowColor="#000000" shadowOffset="-2,-2" zPosition="1" halign="left" >
			<convert type="ServiceTime">StartTime</convert>
			<convert type="ClockToText">Date</convert>
		</widget>

		<widget source="Service" render="Label" position="285,424" size="50,19" foregroundColor="#00bab329" backgroundColor="#270b1b1c" font="Regular;17" shadowColor="#000000" shadowOffset="-2,-2" zPosition="1" halign="left" >
			<convert type="ServiceTime">StartTime</convert>
			<convert type="ClockToText" />
		</widget>

		<widget source="Service" render="Label" position="335,424" size="70,19" foregroundColor="#00bab329" backgroundColor="#270b1b1c" font="Regular;17" shadowColor="#000000" shadowOffset="-2,-2" zPosition="1" halign="left" >
			<convert type="ServiceTime">EndTime</convert>
			<convert type="ClockToText">Format:- %H:%M </convert>
		</widget>

		<widget source="Service" render="Label" position="405,424" size="55,19" foregroundColor="#00bab329" backgroundColor="#270b1b1c" font="Regular;17" shadowColor="#000000" shadowOffset="-2,-2" zPosition="1" halign="left" >
			<convert type="ServiceTime">Duration</convert>
			<convert type="ClockToText">AsLength</convert>
		</widget>

		<widget name="FileSize" position="460,424" size="100,19" foregroundColor="#00bab329" font="Regular;17" backgroundColor="#270b1b1c" shadowColor="#000000" shadowOffset="-2,-2" zPosition="1" halign="left" />
	</screen>"""
	def __init__(self, session):
		Screen.__init__(self, session)
		SelectionEventInfo.__init__(self)
		self.skin = MovieSelectionSP.skin
		self.skinName = "MovieSelectionSP"
		self.wasClosed = True
		self.playerInstance = None
		self.selectIdx = -1
		self.cursorDir = 0

		self.currentGrnText = _("Alpha sort")
		self["list"] = MovieList()
		self["key_red"] = Button()
		self["key_green"] = Button()
		self["key_yellow"] = Button()
		self["key_blue"] = Button()

		self["actions"] = HelpableActionMap(self, "PluginMovieSelectionActions",
			{
				"bOK":		(self.movieSelected,		_("Play selected movie(s)")),
				"bEXIT":	(self.abort, 				_("Close movie list")),
				"bMENU":	(self.openMenu,				_("Open menu")),
				"bINFO":	(self.showEventInformation,	_("Show event info")),
				"bRed":		(self.deleteFile,			_("Delete file or empty dir")),
				"bGreen":	(self.toggleSort,			_("Toggle sort mode")),
				"bYellow":	(self.moveMovie,			_("Move selected movie(s)")),
				"bBlue":	(self.blueFunc,				_("Movie home / Play last (configurable)")),
				"bLeftUp":	(self.keyPress,				_("Move cursor page up")),
				"bRightUp":	(self.keyPress,				_("Move cursor page down")),
				"bUpUp":	(self.keyPressU,			_("Move cursor up")),
				"bDownUp":	(self.keyPressD,			_("Move cursor down")),
				#"bArrowL":	(self.,		("")),
				#"bArrowR":	(self.,		("")),
				"bBqtPlus":	(self.bouquetPlus,			_("Move cursor 1/2 page up")),
				"bBqtMnus":	(self.bouquetMinus,			_("Move cursor 1/2 page down")),
				"bVIDEO":	(self.selectionToggle,		_("Toggle service selection")),
				"bAUDIO":	(self.openMenuPlugins,		_("Available plugins menu")),
				"bTV":		(self.reloadList,			_("Reload movie file list")),
				"bRADIO":	(self.viewedToggle,			_("Toggle viewed / not viewed")),
				"bTEXT":	(self.multiSelect,			_("Start / end multiselection"))
			})
		self["actions"].csel = self
		HelpableScreen.__init__(self)

		self.currentPathSel = config.suomipoeka.movie_homepath.value
		self.parentStack = []
		self.onExecBegin.append(self.onDialogShow)
		DelayedFunction(6000, self.reloadList, -1, False)	# try to read the movie home directory after Enigma is started

	def keyPress(self):		# these only catch the key-Up events, not key-Down or repeat
		self.cursorDir = 0
		DelayedFunction(config.suomipoeka.movie_descdelay.value, self.updateAfterKeyPress)
	def keyPressU(self):
		self.cursorDir = -1
		DelayedFunction(config.suomipoeka.movie_descdelay.value, self.updateAfterKeyPress)
	def keyPressD(self):
		self.cursorDir = 1
		DelayedFunction(config.suomipoeka.movie_descdelay.value, self.updateAfterKeyPress)

	def updateAfterKeyPress(self):
		self.updateTitle()
		if not self.browsingVLC():	self.updateEventInfo()

	def bouquetPlus(self):
		self["list"].moveToIndex( max(self["list"].getCurrentIndex()-8, 0) )

	def bouquetMinus(self):
		self["list"].moveToIndex( min(self["list"].getCurrentIndex()+8, self["list"].__len__()-1) )

	def multiSelect(self):
		if self.browsingVLC(): return
		if self.selectIdx == -1:
			if self["list"].currentSelIsDirectory(): return
			self.selectIdx = self["list"].getCurrentIndex()
		else:
			if self["list"].currentSelIsDirectory() or self.selectIdx==-1: return
			idx = self["list"].getCurrentIndex()
			adder = 1-2*(idx<self.selectIdx)
			for i in range(self.selectIdx, idx+adder, adder): self["list"].multiSelectIdx(i)
			self.selectIdx = -1
		self.updateTitle()

	def selectionToggle(self):
		if not self["list"].currentSelIsDirectory() and not self.browsingVLC():
			self["list"].toggleSelection()
			if config.suomipoeka.movielist_selmove.value:
				if self.cursorDir == -1:
					self["list"].moveToIndex( max(self["list"].getCurrentIndex()-1, 0) )
				else:
					self["list"].moveToIndex( min(self["list"].getCurrentIndex()+1, self["list"].__len__()-1) )

	def viewedToggle(self, service=None, onlyViewed=False):
		try:
			if service is None:
				service = self.getCurrent()
			name = service.getPath()
			if not name.endswith(".ts"): return
			cuts  = name +".cuts"
			cutsr = name +".cutsr"
			if os.path.exists(cuts) and not onlyViewed:
				os.rename(cuts, cutsr)
			elif os.path.exists(cutsr):
				os.rename(cutsr, cuts)
			else:
				open(cuts, 'a').close()
			self["list"].invalidateCurrent()
		except Exception, e:
			spDebugOut("[spML] viewedToggle exception:\n" + str(e))

	def showEventInformation(self):
		from Screens.EventView import EventViewSimple
		from ServiceReference import ServiceReference
		evt = self["list"].getCurrentEvent()
		if evt:
			self.session.open(EventViewSimple, evt, ServiceReference(self.getCurrent()))

	def onDialogShow(self):
		if self.wasClosed:
			self.wasClosed = False
			SuomipoekaAutoremove()
			self.purgeExpiredFromTrash()
			self["key_red"].text = _("Delete")
			self["key_green"].text = self.currentGrnText
			self["key_yellow"].text = _("Move")
			self["key_blue"].text = _(config.suomipoeka.movie_bluefunc.value)
			if config.suomipoeka.movie_reload.value or self["list"].newRecordings or self["list"].__len__() == 0:
				DelayedFunction(80, self.reloadList)
			self.initCursor()
			self.updateTitle()
			try: self.descFieldInit()
			except:	pass
			self.updateEventInfo()

	def initCursor(self):
		if self.playerInstance is None:	# detect movie player state (None == not open)
			if config.suomipoeka.movielist_gotonewest.value:
				self.cursorToNewest()
		else:
			if config.suomipoeka.movielist_gotonewestp.value:
				self.cursorToNewest()
			else:
				service = self.playerInstance.currentlyPlayedMovie()
				if service is not None:
					self["list"].moveTo(service)

	def cursorToNewest(self):
		if config.suomipoeka.movielist_reversed.value:
			self["list"].moveToIndex(self["list"].__len__()-1)
		else:
			self["list"].moveToIndex(0)

	def blueFunc(self):
		if config.suomipoeka.movie_bluefunc.value == "Movie home": self.changeDir(config.suomipoeka.movie_homepath.value)
		elif config.suomipoeka.movie_bluefunc.value == "Play last": self.playLast()

	def changeDir(self, path):
		self.parentStack[:] = []	# clear the list
		self.currentPathSel = path
		self.reloadList()
		self.cursorToNewest()

	def setPlayerInstance(self, player, mayReopen=True):
		try:
			self.playerInstance = player
			if player is None:	# this means that the player was just closed
				if config.suomipoeka.movie_reopen.value and mayReopen:
					self.session.execDialog(self)
		except Exception, e:
			spDebugOut("[spML] setPlayerInstance exception:\n" + str(e))

	def openPlayer(self, playlist):
		if self.playerInstance is None:
			Notifications.AddNotification(MoviePlayerSP, playlist, self)
		else:
			DelayedFunction(10, self.playerInstance.movieSelected, playlist)

	def playLast(self):
		global LastPlayedMovies
		if LastPlayedMovies is None:
			self.session.open(MessageBox, _("Last played movie/playlist not available..."), MessageBox.TYPE_ERROR, 10)
		else:
			self.wasClosed = True
			self.close(None)
			self.openPlayer(LastPlayedMovies)

	def moveTo(self):
		self.updateAfterKeyPress()

	def getCurrent(self):
		return self["list"].getCurrent()

	def delPathSel(self, path):
		if path != "..":
			path = self.currentPathSel+ "/" + path
			if os.path.exists(path):
				if len(os.listdir(path))>0:
					if path == config.suomipoeka.movie_trashpath.value:
						# offer to empty trash
						self.session.openWithCallback(self.emptyTrash, MessageBox, _("Permanently delete all files in trashcan?"), MessageBox.TYPE_YESNO)
					else:
						self.session.open(MessageBox, _("Directory is not empty."), MessageBox.TYPE_ERROR, 10)
				else:
					spTasker.shellExecute('rmdir "' + path +'"')
					return True
		else:
			self.session.open(MessageBox, _("Cannot delete the parent directory."), MessageBox.TYPE_ERROR, 10)
		return False

	def setNextPathSel(self, nextdir):
		moveToIdx = 0
		if nextdir == "..":
			if self.currentPathSel != "" and self.currentPathSel != "/":
				if len(self.parentStack) > 0:
					popped = self.parentStack.pop()
					self.currentPathSel = popped[0]
					moveToIdx = popped[1]
				else:
					# in this case we'll just go to the idx 0
					self.currentPathSel = os.path.split(self.currentPathSel)[0]
		else:
			if os.path.exists(self.currentPathSel+ "/" + nextdir):
				self.parentStack.append((self.currentPathSel, self["list"].getCurrentIndex()))
				if self.currentPathSel == "/": self.currentPathSel = ""
				self.currentPathSel += "/" + nextdir
			elif nextdir == "VLC servers" or self.browsingVLC():
				self.parentStack.append((self.currentPathSel, self["list"].getCurrentIndex()))
				self.currentPathSel += "/" + nextdir
		self.reloadList(moveToIdx)

	def movieSelected(self):
		try:
			global LastPlayedMovies
			current = self.getCurrent()
			if current is not None:
				if self["list"].currentSelIsDirectory():
					self.setNextPathSel( self["list"].getCurrentSelDir() )
				elif self["list"].currentSelIsVlc():
					entry = self["list"].list[ self["list"].getCurrentIndex() ]
					self.hide()
#					from Plugins.Extensions.VlcPlayer.VlcPlayer import VlcPlayer
#					dlg = self.session.open(VlcPlayer, self["list"].vlcServer, VlcFileListWrapper())
#					dlg.playfile(entry[4], entry[3])
					try:	# v2.5
						self["list"].vlcServer.play(self, entry[4], entry[3], VlcFileListWrapper())
					except:	# v2.6
						self["list"].vlcServer.play(self.session, entry[4], entry[3], VlcFileListWrapper())
					self.wasClosed = True
					self.close()
				else:
					playlist = self["list"].makeSelectionList()
					if not self["list"].serviceBusy(playlist[0]):
						LastPlayedMovies = [] + playlist	# force a copy instead of an reference!
						self.wasClosed = True
						self.close()
						self.openPlayer(LastPlayedMovies)
					else:
						self.session.open(MessageBox, _("File not available."), MessageBox.TYPE_ERROR, 10)
		except Exception, e:
			spDebugOut("[spML] movieSelected exception:\n" + str(e))

	def openMenu(self):
		current = self.getCurrent()
		if self["list"].currentSelIsDirectory() or self.browsingVLC(): current = None
		self.session.openWithCallback(self.menuCallback, MovieMenu, (False, current), self.currentPathSel)

	def openMenuPlugins(self):
		current = self.getCurrent()
		if not self["list"].currentSelIsDirectory() and not self.browsingVLC():
			self.session.openWithCallback(self.menuCallback, MovieMenu, (True, current), self.currentPathSel)

	def menuCallback(self, selection=None):
		if selection is not None:
			if selection == "Play last": self.playLast()
			elif selection == "Movie home": self.changeDir(config.suomipoeka.movie_homepath.value)
			elif selection == "reload": self.reloadList()
			elif selection == "ptrash": self.emptyTrash(True)
			elif selection == "trash": self.changeDir(config.suomipoeka.movie_trashpath.value)
			elif selection == "delete": self.deleteFile(True)

	def deleteFile(self, permanently=False):
		if self.browsingVLC(): return
		self.permanentDel  = permanently or config.suomipoeka.movie_trashcan_limit.value == 0 
		self.permanentDel |= self.currentPathSel == config.suomipoeka.movie_trashpath.value
		self.permanentDel |= mountpoint(self.currentPathSel) != mountpoint(config.suomipoeka.movie_trashpath.value)
		current = self.getCurrent()	# make sure there is atleast one entry in the list
		if current is not None:
			selectedlist = self["list"].makeSelectionList()
			if self["list"].currentSelIsDirectory() and len(selectedlist) == 1 and current==selectedlist[0]:
				# try to delete an empty directory
				if self.delPathSel(self["list"].getCurrentSelDir()):
					self["list"].removeService(selectedlist[0])
			else:
				if self["list"].serviceBusy(selectedlist[0]): return
				if selectedlist and len(selectedlist)>0:
					self.recsToStop = []
					for e in selectedlist:
						mpath = e.getPath()
						if self["list"].recControl.isRecording(mpath):
							self.recsToStop.append(mpath)
					if len(self.recsToStop)>0:
						self.stopRecordQ()
					else:
						self.deleteMovieQ(selectedlist)

	def stopRecordQ(self):
		try:
			filenames = ""
			for e in self.recsToStop:
				filenames += "\n" + e.split("/")[-1]
			self.session.openWithCallback(self.stopRecordConfirmation, MessageBox, _("Stop ongoing recording?\n") + filenames, MessageBox.TYPE_YESNO)

		except Exception, e:
			spDebugOut("[spML] stopRecordQ exception:\n" + str(e))

	def stopRecordConfirmation(self, confirmed):
		if not confirmed: return
		# send as a list?
		stoppedAll=True
		for e in self.recsToStop:
			stoppedAll = stoppedAll and self["list"].recControl.stopRecording(e)
		if not stoppedAll:
			self.session.open(MessageBox, _("Not stopping any repeating timers. Modify them with the timer editor."), MessageBox.TYPE_INFO, 10)

	def deleteMovieQ(self, selectedlist):
		try:
			self.moviesToDelete = selectedlist
			self.delCurrentlyPlaying = False

			if self.playerInstance is not None:
				nowPlaying = self.playerInstance.currentlyPlayedMovie()
				if nowPlaying is not None:
					if nowPlaying in selectedlist:
						self.delCurrentlyPlaying = True

			entrycount = len(selectedlist)
			delStr = _("Delete") + _(" permanently")*self.permanentDel
			if entrycount == 1:
				service = selectedlist[0]
				name = "\n\n" + self["list"].getFileNameOfService(service).replace(".ts","")
				if not self.delCurrentlyPlaying:
					self.session.openWithCallback(self.deleteMovieConfimation, MessageBox, delStr + "?" + name, MessageBox.TYPE_YESNO)
				else:
					self.session.openWithCallback(self.deleteMovieConfimation, MessageBox, delStr + _(" currently played?") + name, MessageBox.TYPE_YESNO)
			else:
				if entrycount > 1:
					movienames = "\n\n"
					i = 0
					for service in selectedlist:
						if i >= 5 and entrycount > 5:	# show only 5 entries in the file list
							movienames += "..."
							break
						i += 1
						name = self["list"].getFileNameOfService(service).replace(".ts","")	# TODO: None check
						if len(name) > 48:
							name = name[:48] + "..."	# limit the name string
						movienames += name + "\n"*(i<entrycount)
					if not self.delCurrentlyPlaying:
						self.session.openWithCallback(self.deleteMovieConfimation, MessageBox, delStr + _(" all selected video files?")+movienames, MessageBox.TYPE_YESNO)
					else:
						self.session.openWithCallback(self.deleteMovieConfimation, MessageBox, delStr + _(" all selected video files? The currently playing movie is also one of the selections and its playback will be stopped.")+movienames, MessageBox.TYPE_YESNO)
		except Exception, e:
			self.session.open(MessageBox, _("Delete error:\n") + str(e), MessageBox.TYPE_ERROR)
			spDebugOut("[spML] deleteMovieQ exception:\n" + str(e))

	def deleteMovieConfimation(self, confirmed):
		if confirmed and self.moviesToDelete is not None and len(self.moviesToDelete)>0:
			if self.delCurrentlyPlaying:
				if self.playerInstance is not None:
					self.playerInstance.removeFromPlaylist(self.moviesToDelete)
			delete = config.suomipoeka.movie_trashcan_limit.value==0 or self.permanentDel
			if os.path.exists(config.suomipoeka.movie_trashpath.value) or delete:
				# if the user doesn't want to keep the movies in the trash, purge immediately
				self.execFileOp(config.suomipoeka.movie_trashpath.value, self.moviesToDelete, op="delete", purgeTrash=delete)
				for x in self.moviesToDelete:
					self.lastPlayedCheck(x)
				self["list"].resetSelectionList()
			elif not delete:
				self.session.openWithCallback(self.trashcanCreate, MessageBox, _("Delete failed because the trashcan directory does not exist. Attempt to create it now?"), MessageBox.TYPE_YESNO)
			self.moviesToDelete = None
			self.updateAfterKeyPress()

	def lastPlayedCheck(self, service):
		global LastPlayedMovies
		try:
			if LastPlayedMovies is not None:
				if service in LastPlayedMovies:
					LastPlayedMovies.remove(service)
				if len(LastPlayedMovies) == 0:
					LastPlayedMovies = None
		except Exception, e:
			spDebugOut("[spML] lastPlayedCheck exception:\n" + str(e))

	def abort(self):
		current = self.getCurrent()
		self.wasClosed = True
		self.close(None)
		if self.playerInstance is not None:
			self.playerInstance.movieSelected(None)	# the player is still open and this will close it

	def toggleSort(self):
		if self.browsingVLC(): return
		if self["key_green"].text == _("Alpha sort"):
			self["key_green"].text = _("Date sort")
			self.currentGrnText = _("Date sort")
			self["list"].setAlphaSort(True)
		else:
			self["key_green"].text = _("Alpha sort")
			self.currentGrnText = _("Alpha sort")
			self["list"].setAlphaSort(False)
		self.reloadList()

	def browsingVLC(self):
		return self.currentPathSel.find("VLC servers") > 0

	def reloadList(self, moveToIdx=-1, mayOpenDialog=True):
		self.selectIdx = -1
		try:
			if os.path.exists(self.currentPathSel) or self.browsingVLC():
				self["list"].reload(self.currentPathSel + "/"*(self.currentPathSel != "/"))
				if moveToIdx >= 0: self["list"].moveToIndex(moveToIdx)
				self.updateAfterKeyPress()
			else:
				if mayOpenDialog:
					self.session.open(MessageBox, _("Error: path %s not available!" % self.currentPathSel), MessageBox.TYPE_ERROR)
				else:
					Notifications.AddNotification(MessageBox, _("Error: path %s not available!" % self.currentPathSel), MessageBox.TYPE_ERROR)
		except Exception, e:
			spDebugOut("[spML] reloadList exception:\n" + str(e))

	def updateTitle(self):
		if self.selectIdx != -1:
			self.setTitle(_("*** Multiselection active ***"))
			return
		lotime = localtime()
		title = "[%02d:%02d] " %(lotime[3],lotime[4])
		if os.path.exists(self.currentPathSel+"/"):
			stat = os.statvfs(self.currentPathSel+"/")
			free = (stat.f_bavail if stat.f_bavail!=0 else stat.f_bfree) * stat.f_bsize / 1048576
			if free >= 10240:	#unit in Giga bytes if more than 10 GB free
				title += "(%d GB) " %(free/1024)
			else:
				title += "(%d MB) " %(free)

		title += self.currentPathSel + (self.currentPathSel=="")*"/"

		if not self["list"].currentSelIsDirectory() and not self.browsingVLC():
			try:
				service = self.getCurrent()
				if config.suomipoeka.movie_descdis.value:
					lenght = self["list"].getLenghtOfCurrent()
					title += " <" + ("%d min, "%lenght) * (lenght>0)
					title += "%d MB>" %(os.path.getsize(service.getPath())/1048576)
			except Exception, e:
				spDebugOut("[spML] updateTitle exception:\n" + str(e))
		title = title.replace(config.suomipoeka.movie_homepath.value+"/", ".../")
		self.setTitle(title)

	def moveMovie(self):
		if self.browsingVLC() or self["list"].getCurrentSelDir() == "VLC servers": return
		current = self.getCurrent()
		if current is not None:
			selectedlist = self["list"].makeSelectionList()
			dialog = False
			if self["list"].currentSelIsDirectory():
				if current != selectedlist[0]:	# first selection != cursor pos?
					targetPath = self.currentPathSel
					if self["list"].getCurrentSelDir() == "..":
						targetPath = os.path.split(targetPath)[0]
					else:
						targetPath += "/" + self["list"].getCurrentSelDir()
					self.execFileOp(targetPath, selectedlist)
					self["list"].resetSelectionList()
				else:
					if len(selectedlist) == 1:
						self.session.open(MessageBox, _("How to move files:\nSelect some movies with the VIDEO-button, move the cursor on top of the destination directory and press yellow."), MessageBox.TYPE_ERROR, 10)
					else:
						dialog = True
			else:
				dialog = True
			if dialog:
				try:
					from Screens.LocationBox import LocationBox
					if len(selectedlist)==1 and self["list"].serviceBusy(selectedlist[0]): return
					self.tmpSelList = selectedlist
					self.session.openWithCallback(self.mvDirSelected, LocationBox, windowTitle= _("Select Location"), text = _("Choose directory"),
						filename = "", currDir = self.currentPathSel+"/", minFree = 0)
				except:
					self.session.open(MessageBox, _("How to move files:\nSelect some movies with the VIDEO-button, move the cursor on top of the destination directory and press yellow."), MessageBox.TYPE_ERROR, 10)

	def mvDirSelected(self, targetPath):
		if targetPath is not None:
			self.execFileOp(targetPath, self.tmpSelList)
			self["list"].resetSelectionList()
			self.tmpSelList = None

	def moveRecCheck(self, serviceref, targetPath):
		try:
			path = serviceref.getPath()
			if self["list"].recControl.isRecording(path):
				self["list"].recControl.fixTimerPath(path, path.replace(self.currentPathSel, targetPath))
		except Exception, e:
			spDebugOut("[spML] moveRecCheck exception:\n" + str(e))

	def execFileOp(self, targetPath, selectedlist, op="move", purgeTrash=False):
		mvCmd = ""
		rmCmd = ""
		association = []
		for service in selectedlist:
			name = self["list"].getFileNameOfService(service)
			if name.endswith(".ts"):		name = name[:-3]
			elif name.endswith(".mpg"):		name = name[:-4]
			elif name.endswith(".mpeg"):	name = name[:-5]
			elif name.endswith(".mp3"):		name = name[:-4]
			if name is not None:
				if op=="delete":	# target == trashcan
					if purgeTrash or self.currentPathSel == targetPath or mountpoint(self.currentPathSel) != mountpoint(targetPath):
						# direct delete from the trashcan or network mount (no copy to trashcan from different mountpoint)
						rmCmd += '; rm -f "'+ self.currentPathSel +"/"+ name +'."*'
					else:
						# create a time stamp with touch
						mvCmd += '; touch "'+ self.currentPathSel +"/"+ name +'."*'
						# move movie into the trashcan
						mvCmd += '; mv "'+ self.currentPathSel +"/"+ name +'."* "'+ targetPath +'/"'
					association.append((service, self.delCB))	# put in a callback for this particular movie
					self["list"].highlightService(True, "del", service)
					if config.suomipoeka.movie_hide_del.value:
						self["list"].removeService(service)
				elif op == "move":
					#if mountpoint(self.currentPathSel) == mountpoint(targetPath):	
					#	#self["list"].removeService(service)	# normal direct move
					#	pass
					#else:
					# different mountpoint? -> reset user&group
					if mountpoint(targetPath) != mountpoint(config.suomipoeka.movie_homepath.value):		# CIFS to HDD is ok!
						# need to change file ownership to match target filesystem file creation
						tfile = targetPath + "/owner_test"
						sfile = "\""+ self.currentPathSel +"/"+ name +".\"*"
						mvCmd += "; touch %s;ls -l %s | while read flags i owner group crap;do chown $owner:$group %s;done;rm %s" %(tfile,tfile,sfile,tfile)
					mvCmd += '; mv "'+ self.currentPathSel +"/"+ name +'."* "'+ targetPath +'/"'
					association.append((service, self.moveCB))	# put in a callback for this particular movie
					self["list"].highlightService(True, "move", service)
					if config.suomipoeka.movie_hide_mov.value:
						self["list"].removeService(service)
					self.moveRecCheck(service, targetPath)
				self.lastPlayedCheck(service)
		if (mvCmd + rmCmd) != "":
			spTasker.shellExecute((mvCmd + rmCmd)[2:], association)	# first move, then delete if expiration limit is 0

	def moveCB(self, service):
		self["list"].highlightService(False, "move", service)	# remove the highlight
		if not config.suomipoeka.movie_hide_mov.value:
			self["list"].removeService(service)
		self.updateTitle()

	def delCB(self, service):
		self["list"].highlightService(False, "del", service)	# remove the highlight
		if not config.suomipoeka.movie_hide_del.value:
			self["list"].removeService(service)
		self.updateTitle()

	def emptyTrash(self, confirmed):
		try:
			if confirmed:
				# just make sure the user hasn't done the most stupid thing possible :)
				if not os.path.exists(config.suomipoeka.movie_trashpath.value + "/proc"):
					spTasker.shellExecute("rm -f \"%s/\"*" % config.suomipoeka.movie_trashpath.value)
				else:
					spDebugOut("[spML] emptyTrash WHOA! Trashcan path may be pointing to root, cancelling.")
		except Exception, e:
			spDebugOut("[spML] emptyTrash exception:\n" + str(e))

	def purgeExpiredFromTrash(self):
		try:
			if os.path.exists(config.suomipoeka.movie_trashpath.value):
				purgeCmd = ""
				dirlist = os.listdir(config.suomipoeka.movie_trashpath.value)
				for movie in dirlist:
					if movie[-3:] != ".ts": continue
					fullpath = config.suomipoeka.movie_trashpath.value +"/"+ movie
					currTime = localtime()
					expTime = localtime(os.stat(fullpath).st_mtime + 24*60*60*config.suomipoeka.movie_trashcan_limit.value)

					if currTime > expTime:
						purgeCmd += "; rm -f \"%s\"*" % fullpath.replace(".ts","")
				if purgeCmd != "":
					spTasker.shellExecute(purgeCmd[2:])
		except Exception, e:
			spDebugOut("[spML] purgeExpiredFromTrash exception:\n" + str(e))

	def trashcanCreate(self, confirmed):
		try:
			os.makedirs(config.suomipoeka.movie_trashpath.value)
			self.reloadList()	# reload to show the trashcan
		except Exception, e:
			self.session.open(MessageBox, _("Trashcan create failed. Check mounts and permissions."), MessageBox.TYPE_ERROR)
			spDebugOut("[spML] trashcanCreate exception:\n" + str(e))

