#
# Movie selection interface rewritten as an additional plugin by moveq
# Copyright (C) 2007-2008 moveq / Suomipoeka team (suomipoeka@gmail.com)
# In case of reuse of this source code please do not remove this copyright.
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	For more information on the GNU General Public License see:
#	<http://www.gnu.org/licenses/>.
#

from Components.config import *
from Components.PluginComponent import plugins
from Plugins.Plugin import PluginDescriptor
from Screens.MessageBox import MessageBox
from Screens.FixedMenu import FixedMenu
from Screens.InputBox import InputBox
from Tools.BoundFunction import boundFunction
import os

from SuomipoekaTasker import spDebugOut
from Suomipoeka import _ 

class MovieMenu(FixedMenu):
	def __init__(self, session, intuple, currentPath):
		pluginmode, self.service = intuple[0], intuple[1]
		self.currentPathSel = currentPath

		menu = []
		if not pluginmode:
			if config.suomipoeka.movie_bluefunc.value == "Movie home":
				menu.append((_("Play last"), boundFunction(self.close, "Play last")))
			else:
				menu.append((_("Movie home"), boundFunction(self.close, "Movie home")))

			if os.path.exists(config.suomipoeka.movie_trashpath.value):
				if self.service is not None:
					menu.append((_("Delete permanently"), boundFunction(self.close, "delete")))
				menu.append((_("Empty trashcan"), boundFunction(self.emptyTrash)))
				menu.append((_("Go to trashcan"), boundFunction(self.close, "trash")))
			menu.append((_("Create directory"), boundFunction(self.createDir)))
		elif self.service is not None:
			for p in plugins.getPlugins(PluginDescriptor.WHERE_MOVIELIST):
				menu.append((p.description, boundFunction(self.execPlugin, p)))

		FixedMenu.__init__(self, session, _("Choose operation"), menu)
		self.skinName = "Menu"
		self.onShown.append(self.onDialogShow)

	def onDialogShow(self):
		self.setTitle(_("Movie Selection Menu"))

	def createDir(self):
		self.hide()
		self.session.openWithCallback(self.createDirCB, InputBox, title=_("Enter name for new directory."), windowTitle=_("Create directory"))

	def createDirCB(self, name):
		if name is not None:
			name = self.currentPathSel+ "/" + name
			if os.path.exists(name):
				self.session.open(MessageBox, _("Directory "+name+" already exists."), MessageBox.TYPE_ERROR)
			else:
				try:
					os.mkdir(name)
				except Exception, e:
					spDebugOut("[spML] createDir exception:\n" + str(e))
				self.close("reload")

	def emptyTrash(self):
		self.hide()
		self.session.openWithCallback(self.emptyTrashCB, MessageBox, _("Permanently delete all files in trashcan?"), MessageBox.TYPE_YESNO)

	def emptyTrashCB(self, confirmed):
		if confirmed:
			self.close("ptrash")
		else:
			self.close(None)

	def execPlugin(self, plugin):
		#self.hide()
		plugin(session=self.session, service=self.service)
		#self.close("reload")

