#
# Recordings manipulation class for the movie list by moveq
# Copyright (C) 2007-2008 moveq / Suomipoeka team (suomipoeka@gmail.com)
# In case of reuse of this source code please do not remove this copyright.
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	For more information on the GNU General Public License see:
#	<http://www.gnu.org/licenses/>.
#

from enigma import iRecordableService
from Components.config import config
from RecordTimer import AFTEREVENT
import NavigationInstance
import os

from RecordEventObserver import RecordEventObserver
from SuomipoekaTasker import spDebugOut
from DelayedFunction import DelayedFunction

class RecordingsControl:
	def __init__(self, recStateChange):
		self.recStateChange = recStateChange
		self.recObserver = RecordEventObserver(self.recEvent)
		self.recList = []
		# if there Enigma2 has crashed, we need to recreate the list of the ongoing recordings
		for timer in NavigationInstance.instance.RecordTimer.timer_list:
			self.recEvent(timer)

	def recEvent(self, timer):
		# StateWaiting=0, StatePrepared=1, StateRunning=2, StateEnded=3
		try:
			if timer.justplay: return
			inform = False
			filename = os.path.split(timer.Filename)[1]
#			if timer.state == timer.StatePrepared:
			if timer.state == timer.StateRunning:	# timer.isRunning()
				if not filename in self.recList:
					self.recList.append(filename)
					inform = True
					spDebugOut("[spRC] REC START for " + filename)
			else: #timer.state == timer.StateEnded:
				if filename in self.recList:
					self.recList.remove(filename)
					inform = True
					spDebugOut("[spRC] REC END for " + filename)
				if config.suomipoeka.timer_autocln.value:
					DelayedFunction(2000, NavigationInstance.instance.RecordTimer.cleanup)	# postpone to avoid crash in basic timer delete by user
			if inform:
				self.recStateChange(self.recList)

		except Exception, e:
			spDebugOut("[spRC] recEvent exception:\n" + str(e))

	def isRecording(self, filename):
		try:
			if filename[0] == "/": 			filename = os.path.split(filename)[1]
			if filename.endswith(".ts"):	filename = filename[:-3]
			return filename in self.recList
		except Exception, e:
			spDebugOut("[spRC] isRecording exception:\n" + str(e))
			return False

	def stopRecording(self, filename):
		try:
			if filename[0] == "/":			filename = os.path.split(filename)[1]
			if filename.endswith(".ts"):	filename = filename[:-3]
			if filename in self.recList:
				for timer in NavigationInstance.instance.RecordTimer.timer_list:
					if timer.isRunning() and not timer.justplay and timer.Filename.find(filename)>=0:
						if timer.repeated: return False
						timer.afterEvent = AFTEREVENT.NONE
						NavigationInstance.instance.RecordTimer.removeEntry(timer)
						spDebugOut("[spRC] REC STOP for: " + filename)
						return True
			else:
				spDebugOut("[spRC] OOPS stop REC for nonexistent: " + filename)
		except Exception, e:
			spDebugOut("[spRC] stopRecording exception:\n" + str(e))
		return False

	def fixTimerPath(self, old, new):
		try:
			if old.endswith(".ts"):	old = old[:-3]
			if new.endswith(".ts"):	new = new[:-3]
			for timer in NavigationInstance.instance.RecordTimer.timer_list:
				if timer.isRunning() and not timer.justplay and timer.Filename == old:
					timer.Filename = new
					timer.dirname = os.path.split(new)[0] + "/"
					spDebugOut("[spRC] fixed path: " + new)
					break

		except Exception, e:
			spDebugOut("[spRC] stopRecording exception:\n" + str(e))

