#
# Suomipoeka plugin by moveq
# Copyright (C) 2007-2008 moveq / Suomipoeka team (suomipoeka@gmail.com)
# In case of reuse of this source code please do not remove this copyright.
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	For more information on the GNU General Public License see:
#	<http://www.gnu.org/licenses/>.
#
from Components.config import *
from Components.Button import Button
from Components.ConfigList import ConfigListScreen
from Components.Label import Label
from Components.Language import *
from Screens.MessageBox import MessageBox
from Screens.Screen import Screen
from Screens.ServiceScan import *
#from Screens.Standby import *
import Screens.Standby
from Tools import Notifications
from enigma import eServiceEvent
import os, gettext

from SuomipoekaTasker import spTasker, spOutputDir, spDebugOut, spStandbyIdFile

SuomipoekaVersion = "v0.88"
SuomipoekaAbout = "Suomipoeka plugin " +SuomipoekaVersion+ " by moveq. Aimed to enhance the recordings list user experience and also to give better localisation support with different GUI & EPG languages.\n\nIncase of any reuse of this work in any image please give credit.\n\nContact at: suomipoeka@gmail.com"

from Components.Language import language
def setEPGLanguage():
	if config.suomipoeka.epglang.value:
	 	eServiceEvent.setEPGLanguage(config.suomipoeka.epglang.value)
language.addCallback(setEPGLanguage)

def localeInit():
	from Tools.Directories import resolveFilename, SCOPE_PLUGINS, SCOPE_LANGUAGE
	lang = language.getLanguage()[:2] # getLanguage returns e.g. "fi_FI" for "language_country"
	os.environ["LANGUAGE"] = lang # Enigma doesn't set this (or LC_ALL, LC_MESSAGES, LANG). gettext needs it!
#	gettext.bindtextdomain("enigma2", resolveFilename(SCOPE_LANGUAGE))
#	gettext.textdomain("enigma2")
	gettext.bindtextdomain("Suomipoeka", resolveFilename(SCOPE_PLUGINS, "Extensions/Suomipoeka/locale"))

def _(txt):
	if language.getActiveLanguage() == "en_EN": return txt
	return gettext.dgettext("Suomipoeka", txt)

def setupKeyResponseValues(dummy=None):
	spTasker.shellExecute("echo %x >/proc/stb/ir/rc/period" % config.suomipoeka.key_period.value)
	spTasker.shellExecute("echo %x >/proc/stb/ir/rc/repeat" % config.suomipoeka.key_repeat.value)

def movieDirPermissions(dummy=None):
	if config.suomipoeka.movie_perm.value:
		spTasker.shellExecute("chmod %d -R /media/hdd/movie" % config.suomipoeka.movie_perm_val.value)

def SuomipoekaAutoremove():
	# Suomipoeka option for deleting the extra meta data files
	if config.suomipoeka.autochange_cifs.value == "u":
		spTasker.shellExecute("find %s -name cifs* | grep -v '.ts' | while read fn; do mv $fn $fn.ts; done" % config.suomipoeka.movie_homepath.value)
	elif config.suomipoeka.autochange_cifs.value == "d":
		spTasker.shellExecute("find %s -name cifs* | grep -v '.ts' | while read fn; do rm -f $fn; done" % config.suomipoeka.movie_homepath.value)

def suomipoekaStartup(session):
	if not os.path.exists(spOutputDir):
		spTasker.shellExecute("mkdir " + spOutputDir)
	spDebugOut("+++ Suomipoeka "+SuomipoekaVersion+" startup")

 	if config.suomipoeka.epglang.value:
	 	eServiceEvent.setEPGLanguage(config.suomipoeka.epglang.value)
	setupKeyResponseValues()
	movieDirPermissions()

	# Go into standby if the reason for restart was Suomipoeka auto-restart
	if os.path.exists(spStandbyIdFile):
		spDebugOut("+++ Going into Standby mode after auto-restart")
		Notifications.AddNotification(Screens.Standby.Standby)
		spTasker.shellExecute("mv -f " + spStandbyIdFile + " " + spStandbyIdFile + ".old")


class SuomipoekaMenu(ConfigListScreen, Screen):
	skin = """
	<screen name="SuomipoekaMenu" position="75,90" size="560,440" title="SuomipoekaMenu">
	  <ePixmap pixmap="/usr/lib/enigma2/python/Plugins/Extensions/Suomipoeka/img/key-red.png" position="0,0" zPosition="0" size="140,40" transparent="1" alphatest="on" />
	  <ePixmap pixmap="/usr/lib/enigma2/python/Plugins/Extensions/Suomipoeka/img/key-green.png" position="140,0" zPosition="0" size="140,40" transparent="1" alphatest="on" />
	  <ePixmap pixmap="/usr/lib/enigma2/python/Plugins/Extensions/Suomipoeka/img/key-yellow.png" position="280,0" zPosition="0" size="140,40" transparent="1" alphatest="on" />
	  <ePixmap pixmap="/usr/lib/enigma2/python/Plugins/Extensions/Suomipoeka/img/key-blue.png" position="420,0" zPosition="0" size="140,40" transparent="1" alphatest="on" />
	  <widget name="key_red" position="0,0" zPosition="1" size="140,40"
		font="Regular;20" valign="center" halign="center" backgroundColor="#9f1313" transparent="1"
		shadowColor="#000000" shadowOffset="-1,-1" />
	  <widget name="key_green" position="140,0" zPosition="1" size="140,40"
		font="Regular;20" valign="center" halign="center" backgroundColor="#1f771f" transparent="1"
		shadowColor="#000000" shadowOffset="-1,-1" />
	  <widget name="key_yellow" position="280,0" zPosition="1" size="140,40"
		font="Regular;20" valign="center" halign="center" backgroundColor="#a08500" transparent="1"
		shadowColor="#000000" shadowOffset="-1,-1" />
	  <widget name="key_blue" position="420,0" zPosition="1" size="140,40"
		font="Regular;20" valign="center" halign="center" backgroundColor="#18188b" transparent="1"
		shadowColor="#000000" shadowOffset="-1,-1" />
	  <widget name="config" position="10,40" size="540,400" scrollbarMode="showOnDemand" />
	</screen>"""

	def __init__(self, session):
		Screen.__init__(self, session)
		self.skinName = "SuomipoekaMenu"
		self.skin = SuomipoekaMenu.skin

		self["actions"] = ActionMap(["OkCancelActions", "ColorActions"],
		{
			"ok": self.keyOK,
			"cancel": self.keyCancel,
			"red": self.keyCancel,
			"green": self.keySaveNew,
		}, -2)

		self["key_red"] = Button(_("Cancel"))
		self["key_green"] = Button(_("Save"))
		self["key_yellow"] = Button(" ")
		self["key_blue"] = Button(" ")

		self.list = []
		ConfigListScreen.__init__(self, self.list)

		self.onShown.append(self.onDialogShow)

		self.list.append(getConfigListEntry(_("About"), config.suomipoeka.about, None, 0))
		self.list.append(getConfigListEntry(_("Button to override for MovieList launch"), config.suomipoeka.movie_launch, self.launchListSet, 35))
		self.list.append(getConfigListEntry(_("Show plugin config in extensions menu"), config.suomipoeka.extmenu_plugin, self.needsRestart, 32))
		self.list.append(getConfigListEntry(_("Show MovieList in extensions menu"), config.suomipoeka.extmenu_list, self.needsRestart, 32))
		self.list.append(getConfigListEntry(_("Preferred EPG language"), config.suomipoeka.epglang, None, 1))
#		self.list.append(getConfigListEntry(_("GUI transparency (0-99)"), config.suomipoeka.dialog_transparency, self.transparencyChange, 2))
		self.list.append(getConfigListEntry(_("Movie home auto-update perms (chmod)"), config.suomipoeka.movie_perm, movieDirPermissions, 16))
		self.list.append(getConfigListEntry(_("Movie home chmod permission value"), config.suomipoeka.movie_perm_val, movieDirPermissions, 17))
		self.list.append(getConfigListEntry(_("Movie home home path"), config.suomipoeka.movie_homepath, self.validatePath, 13))
		self.list.append(getConfigListEntry(_("Movie trashcan path"), config.suomipoeka.movie_trashpath, self.validatePath, 13))
#		self.list.append(getConfigListEntry(_("Default recording path"), config.suomipoeka.movie_defpath, self.validatePath, 13))
		self.list.append(getConfigListEntry(_("How many days files may remain in trashcan"),config.suomipoeka.movie_trashcan_limit, None, 31))
		self.list.append(getConfigListEntry(_("MovieList path access limit"), config.suomipoeka.movie_pathlimit, self.validatePath, 13))
		self.list.append(getConfigListEntry(_("MovieList disable desc field"), config.suomipoeka.movie_descdis, None, 18))
		self.list.append(getConfigListEntry(_("MovieList desc field update delay"), config.suomipoeka.movie_descdelay, None, 29))
		self.list.append(getConfigListEntry(_("MovieList file order reverse"), config.suomipoeka.movielist_reversed, None, 20))
		self.list.append(getConfigListEntry(_("MovieList open with cursor on newest (TV mode)"), config.suomipoeka.movielist_gotonewest, None, 26))
		self.list.append(getConfigListEntry(_("MovieList open with cursor on newest (player)"), config.suomipoeka.movielist_gotonewestp, None, 27))
		self.list.append(getConfigListEntry(_("MovieList cursor predictive move after selection"), config.suomipoeka.movielist_selmove, None, 40))
		self.list.append(getConfigListEntry(_("MovieList file dates on left side"), config.suomipoeka.movie_dateleft, None, 25))
		self.list.append(getConfigListEntry(_("MovieList show movie icons"), config.suomipoeka.movie_icons, None, 36))
		self.list.append(getConfigListEntry(_("MovieList show icon indication for non-watched"), config.suomipoeka.movie_mark, None, 36))
		self.list.append(getConfigListEntry(_("MovieList try to load titles from .meta files"), config.suomipoeka.movie_metaload, None, 37))
		self.list.append(getConfigListEntry(_("MovieList re-open list after STOP-press"), config.suomipoeka.movie_reopen, None, 12))
		self.list.append(getConfigListEntry(_("MovieList always reload after open"), config.suomipoeka.movie_reload, None, 28))
		self.list.append(getConfigListEntry(_("MovieList blue button function"), config.suomipoeka.movie_bluefunc, None, 19))
		self.list.append(getConfigListEntry(_("MovieList cifs* files operation"), config.suomipoeka.autochange_cifs, None, 21))
		self.list.append(getConfigListEntry(_("MovieList hide movies being moved"), config.suomipoeka.movie_hide_mov, None, 33))
		self.list.append(getConfigListEntry(_("MovieList hide movies being deleted"), config.suomipoeka.movie_hide_del, None, 33))
		self.list.append(getConfigListEntry(_("MovieList hide trashcan directory"), config.suomipoeka.movie_trashcan_hide, None, 30))
		self.list.append(getConfigListEntry(_("MoviePlayer advanced seek start value"), config.suomipoeka.player_seek, None, 38))
		self.list.append(getConfigListEntry(_("MoviePlayer advanced seek change %"), config.suomipoeka.player_seekchg, None, 39))

#		self.list.append(getConfigListEntry(_("Show record started notification"), config.suomipoeka.rec_start_notify, None, 34))
#		self.list.append(getConfigListEntry(_("Confirm timer delete from user"), config.suomipoeka.timer_delq, None, 22))
#		self.list.append(getConfigListEntry(_("Confirm timers cleanup from user"), config.suomipoeka.timer_clnq, None, 23))
		self.list.append(getConfigListEntry(_("Automatic timers list cleaning"), config.suomipoeka.timer_autocln, None, 24))

#		self.list.append(getConfigListEntry(_("Execute ntpdate after Enigma start"), config.suomipoeka.nptd_init, self.ntpdateChange, 11))
#		self.list.append(getConfigListEntry(_("Server for ntpdate"), config.suomipoeka.nptd_serv, self.ntpdateChange, 10))
		self.list.append(getConfigListEntry(_("Enigma daily auto-restart"), config.suomipoeka.enigmarestart, None, 5))
		self.list.append(getConfigListEntry(_("Enigma auto-restart window begin"), config.suomipoeka.enigmarestart_begin, None, 6))
		self.list.append(getConfigListEntry(_("Enigma auto-restart window end"), config.suomipoeka.enigmarestart_end, None, 7))
		self.list.append(getConfigListEntry(_("Force standby after auto-restart"), config.suomipoeka.enigmarestart_stby, None, 8))
		self.list.append(getConfigListEntry(_("Suomipoeka debug output to /hdd/suomipoeka/"), config.suomipoeka.debug, self.dbgChange, 9))
		self.list.append(getConfigListEntry(_("Debug output file name"), config.suomipoeka.debugfile, None, 501))
		self.list.append(getConfigListEntry(_("Key period value (30-900)"), config.suomipoeka.key_period, setupKeyResponseValues, 3))
		self.list.append(getConfigListEntry(_("Key repeat value (250-900)"), config.suomipoeka.key_repeat, setupKeyResponseValues, 4))
		try:
			self.list.append(getConfigListEntry(_("Enable component video in A/V Settings"), config.av.yuvenabled, self.needsRestart, 500))
		except: pass

		self.needsRestartFlag = False

	def onDialogShow(self):
		self.setTitle("Suomipoeka! "+ SuomipoekaVersion)

	def needsRestart(self, dummy=None):
		self.needsRestartFlag = True

	def launchListSet(self, value):
		if value is not None:
			self.needsRestart()

	def ntpdateChange(self, value):
		if config.suomipoeka.nptd_init.value == True:
			server = config.suomipoeka.nptd_serv.value
			spTasker.shellExecute("echo ntpdate " + server + ">/home/root/ntpdate.sh; chmod 744 /home/root/ntpdate.sh")
		else:
			spTasker.shellExecute("rm -f /home/root/ntpdate.sh")

	def transparencyChange(self, value):
		valstr = "%02x" % (value*200/99)
		spTasker.shellExecute("sh /home/root/suomipoeka.sh transparency " + valstr)
		self.session.open(MessageBox, _("Transparency change will take effect after Enigma restart."), MessageBox.TYPE_INFO, 5)

	def dbgChange(self, value):
		if value == True:
			pass
		else:
			spTasker.shellExecute("rm -f " + spOutputDir + config.suomipoeka.debugfile.value)

	def validatePath(self, value):
		if not os.path.exists(str(value)):
			self.session.open(MessageBox, _("Given path %s does not exist. Please change." % str(value)), MessageBox.TYPE_ERROR)

	def keyOK(self, redirect=0):
		selection = self.list[self["config"].getCurrentIndex()][3]
		if redirect != 0:
			selection = redirect

		if selection == 0:	# self.list[current][3] == 0
			self.session.open(MessageBox, SuomipoekaAbout, MessageBox.TYPE_INFO)
		elif selection == 5:
			spTasker.ShowAutoRestartInfo()
		elif selection == 13: #and selection <= 15:
			try:
				from Screens.LocationBox import LocationBox
				self.session.openWithCallback(self.dirSelected, LocationBox, text = _("Choose directory"),
					filename = "", currDir = config.suomipoeka.movie_homepath.value+"/", minFree = 100) # TODO: fix FileList to correctly determine mountpoint (-ritzMo)
			except:
				pass

	def dirSelected(self, res):
		if res is not None:
			if res[-1:] == "/":
				res = res[:-1]
			self.list[self["config"].getCurrentIndex()][1].value = res

	def keySaveNew(self):
		for entry in self.list:
			if entry[1].isChanged():
				entry[1].save()
				if entry[2] is not None:
					entry[2](entry[1].value)	# execute value changed -function
		if self.needsRestartFlag:
			self.session.open(MessageBox, _("Some settings changes require GUI restart to take effect."), MessageBox.TYPE_INFO, 10)
		self.keySave()
		self.close()

	def keyCancel(self):
		for entry in self.list:
			if entry[1].isChanged():
				entry[1].cancel()
		self.close()

